#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>   // gettimeofday
#include <time.h>       // clock_gettime

long get_real_time_usec() {
	struct timeval tv;
	gettimeofday(&tv, NULL);
    return tv.tv_sec*1000000 + tv.tv_usec;
}

long get_real_time_nsec() {
	struct timespec ts;
	clock_gettime(CLOCK_MONOTONIC, &ts);
    return ts.tv_sec*1000000000+ts.tv_nsec;
}

// Since Linux 2.6.32; Linux-specific
#ifdef __linux__
long get_real_time_nsec_coarse() {
	struct timespec ts;
	clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
    return ts.tv_sec*1000000000+ts.tv_nsec;
}
#endif

#define N 1000

void measure_usec(long *buf, unsigned n) {
    int i;
    printf("-------------------------------------------------\n");
    printf("Sampling usec...\n");
    for (i=0; i+3 < n; i+=4) {
        buf[i]   = get_real_time_usec();
        buf[i+1] = get_real_time_usec();
        buf[i+2] = get_real_time_usec();
        buf[i+3] = get_real_time_usec();
    }
    printf("Sampled values:\n");
    for (i=0; i<n; ++i)
        printf("%-5d: %lu usec\n", i, buf[i]);
}

void measure_nsec(long *buf, unsigned n) {
    int i;
    printf("-------------------------------------------------\n");
    printf("Sampling nsec...\n");
    for (i=0; i+3 < n; i+=4) {
        buf[i]   = get_real_time_nsec();
        buf[i+1] = get_real_time_nsec();
        buf[i+2] = get_real_time_nsec();
        buf[i+3] = get_real_time_nsec();
    }
    printf("Sampled values:\n");
    for (i=0; i<n; ++i)
        printf("%-5d: %lu nsec\n", i, buf[i]);
}

#ifdef __linux__
void measure_nsec_coarse(long *buf, unsigned n) {
    int i;
    printf("-------------------------------------------------\n");
    printf("Sampling nsec [coarse]...\n");
    for (i=0; i+3 < n; i+=4) {
        buf[i]   = get_real_time_nsec_coarse();
        buf[i+1] = get_real_time_nsec_coarse();
        buf[i+2] = get_real_time_nsec_coarse();
        buf[i+3] = get_real_time_nsec_coarse();
    }
    printf("Sampled values:\n");
    for (i=0; i<n; ++i)
        printf("%-5d: %lu nsec\n", i, buf[i]);
}
#endif

int main() {
    long *buf = malloc(N*sizeof(long));
    #ifdef __linux__
    measure_nsec_coarse(buf, N);
    #endif
    measure_nsec(buf, N);
    measure_usec(buf, N);
    free(buf);
    return EXIT_SUCCESS;
}
