#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>   // gettimeofday
#include <time.h>       // clock_gettime

long get_real_time_nsec() {
	struct timespec ts;
	clock_gettime(CLOCK_MONOTONIC, &ts);
    return ts.tv_sec*1000000000+ts.tv_nsec;
}

#define N 1000000

void measure_usec(unsigned n) {
    int i;
	struct timeval tv;
    printf("-------------------------------------------------\n");
    printf("Measuring gettimeofday...\n");
    long start = get_real_time_nsec();
    for (i=0; i+3 < n; i+=4) {
        gettimeofday(&tv, NULL);
        gettimeofday(&tv, NULL);
        gettimeofday(&tv, NULL);
        gettimeofday(&tv, NULL);
    }
    long elapsed = get_real_time_nsec()-start;
    printf("Time per gettimeofday: %f nsec\n", (double)elapsed/n);
}

void measure_nsec(unsigned n) {
    int i;
	struct timespec ts;
    printf("-------------------------------------------------\n");
    printf("Measuring clock_gettime...\n");
    long start = get_real_time_nsec();
    for (i=0; i+3 < n; i+=4) {
        clock_gettime(CLOCK_MONOTONIC, &ts);
        clock_gettime(CLOCK_MONOTONIC, &ts);
        clock_gettime(CLOCK_MONOTONIC, &ts);
        clock_gettime(CLOCK_MONOTONIC, &ts);
    }
    long elapsed = get_real_time_nsec()-start;
    printf("Time per clock_gettime: %f nsec\n", (double)elapsed/n);
}

// Since Linux 2.6.32; Linux-specific
#ifdef __linux__
void measure_nsec_coarse(unsigned n) {
    int i;
	struct timespec ts;
    printf("-------------------------------------------------\n");
    printf("Measuring clock_gettime [coarse]...\n");
    long start = get_real_time_nsec();
    for (i=0; i+3 < n; i+=4) {
        clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
        clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
        clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
        clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
    }
    long elapsed = get_real_time_nsec()-start;
    printf("Time per clock_gettime: %f nsec\n", (double)elapsed/n);
}
#endif

int main() {
    measure_usec(N);
    measure_nsec(N);
    #ifdef __linux__
    measure_nsec_coarse(N);
    #endif
    return EXIT_SUCCESS;
}
